Option Strict Off
Option Explicit On
Friend Class HugeBinaryFile
	'
	'HugeBinaryFile
	'==============
	'Based on an old MS KB article for VB4
	'
	'A class for doing simple binary I/O on very large disk files
	'(well over the usual 2GB limit).  It only does I/O using Byte
	'arrays, and makes use of Currency values that are scaled to
	'whole numbers in places:
	'
	'   For a file of one byte the FileLen property returns 1.0000 as
	'   its value.
	'
	'Operation is similar in many ways to native VB Get#/Put# I/O, for
	'example the EOF property must be checked after a ReadBytes() call.
	'You must also Dim/Redim buffers to desired sizes before calling
	'ReadBytes() or WriteBytes().
	'
	'Short (signed Long) relative seeks and long (unsigned Currency)
	'absolute seeks from 0 may be done.
	'
	'AutoFlush may be set True to force buffer flushes on every write.
	'The Flush() method may be called explicitly if necessary.
	'
	'*************************************************
	'*
	'*  Modified by TheyKilledKenny on 17 Oct 2019
	'*
	'*************************************************
	
	
	
	Public Enum HBF_Errors
		HBF_UNKNOWN_ERROR = 45600
		HBF_FILE_ALREADY_OPEN
		HBF_OPEN_FAILURE
		HBF_SEEK_FAILURE
		HBF_FILELEN_FAILURE
		HBF_READ_FAILURE
		HBF_WRITE_FAILURE
		HBF_FILE_ALREADY_CLOSED
	End Enum
	
	Private Const HBF_SOURCE As String = "EmuTool"
	
    Private Declare Function FormatMessage Lib "kernel32" Alias "FormatMessageA" (ByVal dwFlags As Integer, ByRef lpSource As Integer, ByVal dwMessageId As Integer, ByVal dwLanguageId As Integer, ByVal lpBuffer As String, ByVal nSize As Integer, ByRef Arguments As Long) As Integer
	
	Private hFile As Integer
	Private sFName As String
	Private fAutoFlush As Boolean
	Private fEOF As Boolean
    Private C As ULong
	Private L As Cur2Long
	
	
	Public Property AutoFlush() As Boolean
		Get
			RaiseErrorIfClosed()
			AutoFlush = fAutoFlush
		End Get
		Set(ByVal Value As Boolean)
			RaiseErrorIfClosed()
			fAutoFlush = Value
		End Set
	End Property
	
	Public ReadOnly Property FileHandle() As Integer
		Get
			RaiseErrorIfClosed()
			FileHandle = hFile
		End Get
	End Property
	
	'UPGRADE_NOTE: FileLen  stato aggiornato a FileLen_Renamed. Fare clic per ulteriori informazioni: 'ms-help://MS.VSCC.v90/dv_commoner/local/redirect.htm?keyword="A9E4979A-37FA-4718-9994-97DD76ED70A7"'
	Public ReadOnly Property FileLen_Renamed() As Decimal
		Get
			RaiseErrorIfClosed()
			L.LowVal = GetFileSize(hFile, L.HighVal)
			If L.LowVal = INVALID_FILE_SIZE Then
				If Err.LastDllError Then RaiseError(HBF_Errors.HBF_FILELEN_FAILURE)
				L.LowVal = 0
				L.HighVal = 0
			End If
            Int2Long(L.LowVal, L.HighVal, C)
            'C = LSet(L)
            FileLen_Renamed = C '.Value * 10000@
		End Get
	End Property
	
	Public ReadOnly Property FileName() As String
		Get
			RaiseErrorIfClosed()
			FileName = sFName
		End Get
	End Property
	
	'UPGRADE_NOTE: EOF  stato aggiornato a EOF_Renamed. Fare clic per ulteriori informazioni: 'ms-help://MS.VSCC.v90/dv_commoner/local/redirect.htm?keyword="A9E4979A-37FA-4718-9994-97DD76ED70A7"'
	Public ReadOnly Property EOF_Renamed() As Boolean
		Get
			RaiseErrorIfClosed()
			EOF_Renamed = fEOF
		End Get
	End Property
	
	Public ReadOnly Property IsOpen() As Boolean
		Get
			IsOpen = hFile <> INVALID_HANDLE_VALUE
		End Get
	End Property
	
	Public Sub CloseFile()
		RaiseErrorIfClosed()
		CloseHandle(hFile)
		sFName = ""
		fAutoFlush = False
		fEOF = False
		hFile = INVALID_HANDLE_VALUE
	End Sub
	
	Public Sub Flush()
		RaiseErrorIfClosed()
		FlushFileBuffers(hFile)
	End Sub
	
	'Added the flag to open a file only if it Exist (not create)
	Public Function OpenFile(ByVal OpenFileName As String, Optional ByVal bOpenExisting As Boolean = False) As Boolean
		
		If hFile <> INVALID_HANDLE_VALUE Then
			MsgBox(RaiseError(HBF_Errors.HBF_FILE_ALREADY_OPEN), MsgBoxStyle.OKOnly, LangLabel("Errore", "Errore!"))
			OpenFile = False
		Else
			
			If bOpenExisting Then
				hFile = CreateFile(OpenFileName, GENERIC_WRITE Or GENERIC_READ, 0, 0, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0)
			Else
				hFile = CreateFile(OpenFileName, GENERIC_WRITE Or GENERIC_READ, 0, 0, OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL, 0)
			End If
			
			
			If hFile = INVALID_HANDLE_VALUE Then
				sFName = RaiseError(HBF_Errors.HBF_OPEN_FAILURE) ', vbOKOnly, LangLabel("Errore", "Errore!")
				OpenFile = False
			Else
				OpenFile = True
				sFName = OpenFileName
			End If
			
		End If
		
		
EXIT_SUB: 
		
		Exit Function
		
ERR_SUB: 
		
		MsgBox(Err.Number & " - " & Err.Description & "(" & OpenFileName & ")")
		Resume EXIT_SUB
		
	End Function
	
	Public Function ReadBytes(ByRef Buffer() As Byte) As Integer
		RaiseErrorIfClosed()
		If ReadFile(hFile, Buffer(LBound(Buffer)), UBound(Buffer) - LBound(Buffer) + 1, ReadBytes, 0) Then
			If ReadBytes = 0 Then
				fEOF = True
			End If
		Else
			RaiseError(HBF_Errors.HBF_READ_FAILURE)
		End If
	End Function
	
    Public Sub SeekAbsolute(ByVal Position As ULong)
        RaiseErrorIfClosed()
        'C.Value = Position / 10000@
        C = Position

        'L = LSet(C)

        Long2Int(Position, L.LowVal, L.HighVal)

        If SetFilePointer(hFile, L.LowVal, L.HighVal, FILE_BEGIN) = INVALID_SET_FILE_POINTER Then
            If Err.LastDllError Then RaiseError(HBF_Errors.HBF_SEEK_FAILURE)
        End If

    End Sub
	
	Public Sub SeekEnd()
		RaiseErrorIfClosed()
		If SetFilePointer(hFile, 0, 0, FILE_END) = INVALID_SET_FILE_POINTER Then
			RaiseError(HBF_Errors.HBF_SEEK_FAILURE)
		End If
	End Sub
	
	Public Sub SeekRelative(ByVal Offset As Integer)
		'Offset is signed.
		RaiseErrorIfClosed()
		If SetFilePointer(hFile, Offset, 0, FILE_CURRENT) = INVALID_SET_FILE_POINTER Then
			RaiseError(HBF_Errors.HBF_SEEK_FAILURE)
		End If
	End Sub
	
	Public Function WriteBytes(ByRef Buffer() As Byte) As Integer
		RaiseErrorIfClosed()
		If WriteFile(hFile, Buffer(LBound(Buffer)), UBound(Buffer) - LBound(Buffer) + 1, WriteBytes, 0) Then
			If fAutoFlush Then Flush()
		Else
			RaiseError(HBF_Errors.HBF_WRITE_FAILURE)
		End If
	End Function
	
	Private Sub Class_Initialize_Renamed()
		hFile = INVALID_HANDLE_VALUE
	End Sub
	Public Sub New()
		MyBase.New()
		Class_Initialize_Renamed()
	End Sub
	
	'UPGRADE_NOTE: Class_Terminate  stato aggiornato a Class_Terminate_Renamed. Fare clic per ulteriori informazioni: 'ms-help://MS.VSCC.v90/dv_commoner/local/redirect.htm?keyword="A9E4979A-37FA-4718-9994-97DD76ED70A7"'
	Private Sub Class_Terminate_Renamed()
		If hFile <> INVALID_HANDLE_VALUE Then CloseHandle(hFile)
	End Sub
	Protected Overrides Sub Finalize()
		Class_Terminate_Renamed()
		MyBase.Finalize()
	End Sub
	
	Private Function RaiseError(ByVal ErrorCode As HBF_Errors) As String
		
		Dim Win32Err As Integer
		Dim Win32Text As String
		
		Win32Err = Err.LastDllError
		If Win32Err Then
			Win32Text = vbNewLine & "Error " & Win32Err & vbNewLine & DecodeAPIErrors(Win32Err)
		End If
		If IsOpen Then CloseFile()
		
		On Error GoTo ERR_SUB
		
		Select Case ErrorCode
			Case HBF_Errors.HBF_FILE_ALREADY_OPEN
				RaiseError = CStr(HBF_Errors.HBF_FILE_ALREADY_OPEN) & "- File already open "
			Case HBF_Errors.HBF_OPEN_FAILURE
				RaiseError = CStr(HBF_Errors.HBF_OPEN_FAILURE) & " - Error opening file. " & Win32Text
			Case HBF_Errors.HBF_SEEK_FAILURE
				RaiseError = CStr(HBF_Errors.HBF_SEEK_FAILURE) & " - Seek Error. " & Win32Text
			Case HBF_Errors.HBF_FILELEN_FAILURE
				RaiseError = CStr(HBF_Errors.HBF_FILELEN_FAILURE) & " - GetFileSize Error. " & Win32Text
			Case HBF_Errors.HBF_READ_FAILURE
				RaiseError = CStr(HBF_Errors.HBF_READ_FAILURE) & " - Read failure. " & Win32Text
			Case HBF_Errors.HBF_WRITE_FAILURE
				RaiseError = CStr(HBF_Errors.HBF_WRITE_FAILURE) & " - Write failure. " & Win32Text
			Case HBF_Errors.HBF_FILE_ALREADY_CLOSED
				RaiseError = CStr(HBF_Errors.HBF_FILE_ALREADY_CLOSED) & " - File must be open for this operation"
			Case Else
				RaiseError = CStr(HBF_Errors.HBF_UNKNOWN_ERROR) & " - Unknown error " & Win32Text
		End Select
		
		
EXIT_SUB: 
		
		Exit Function
		
ERR_SUB: 
		
		
	End Function
	
	Private Sub RaiseErrorIfClosed()
		If hFile = INVALID_HANDLE_VALUE Then RaiseError(HBF_Errors.HBF_FILE_ALREADY_CLOSED)
	End Sub
	
	Private Function DecodeAPIErrors(ByVal ErrorCode As Integer) As String
		Const FORMAT_MESSAGE_FROM_SYSTEM As Integer = &H1000
		Dim strMsg As String
		Dim lngMsgLen As Integer
		
		strMsg = Space(256)
		lngMsgLen = FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, 0, ErrorCode, 0, strMsg, 256, 0)
		If lngMsgLen > 0 Then
			DecodeAPIErrors = Left(strMsg, lngMsgLen)
		Else
			DecodeAPIErrors = "Unknown Error."
		End If
	End Function
	

	
	
	'*************************************************
	'*
	'*  Added by TheyKilledKenny on 17 Oct 2019
	'*
	'*************************************************
	Public Function readDataToFileId(ByRef lStartByte As Decimal, ByVal lNumOfBytes As Decimal, ByVal cDstStartByte As Decimal, ByRef HF As HugeBinaryFile) As Decimal
		
		Dim iSecNum As Short
		Dim Buffer() As Byte
		Dim res As Integer
		
		Dim BytesToWrite As Decimal 'SecToWrite
		Dim RemainingBytes As Decimal
		Dim WrittenSectors As Decimal
		Dim idx As Integer
		
		Dim MaxSector As Decimal
		MaxSector = 65535@ * 512@ ' / 10000@
		
		
		On Error GoTo ERR_FUNCT
		
		RemainingBytes = lNumOfBytes
		WrittenSectors = 0
		
		
		Do 
			If (RemainingBytes > MaxSector) Then
				BytesToWrite = MaxSector
				RemainingBytes = RemainingBytes - MaxSector
			Else
				BytesToWrite = RemainingBytes
				RemainingBytes = 0
			End If
			
			'res = DirectReadDriveNT(drvDeviceID, lStartSect + WrittenSectors, Buffer(), EMU_BLOCK_SIZE * (SecToWrite * 10000@))
			ReDim Buffer(BytesToWrite - 1)
			
			SeekAbsolute((lStartByte + WrittenSectors))
			res = ReadBytes(Buffer)
			
			
			If res > 0 Then
				
				HF.SeekAbsolute((cDstStartByte + WrittenSectors))
				HF.WriteBytes(Buffer)
				
				WrittenSectors = WrittenSectors + BytesToWrite
				
				lblStatus.Text = "Written blocks: " & VB6.Format(CStr(WrittenSectors / 512), "#,###")
				System.Windows.Forms.Application.DoEvents()
				
				If g_bStopOperations Then Exit Function
				
			Else
				MsgBox(LangLabel("ErroreLettura", "Error reading disk, start EmuTool in Administrative mode"))
				Exit Do
			End If
			
		Loop While RemainingBytes > 0
		
		
		
EXIT_FUNCT: 
		
		readDataToFileId = WrittenSectors
		Exit Function
		
ERR_FUNCT: 
		
		'    If Err.Number = 1 Then
		'        Resume
		'    Else
		MsgBox(LangLabel("Errore", "Errore") & ": " & Err.Number & " - " & Err.Description)
		Resume EXIT_FUNCT
		'    End If
		
    End Function


    'Replaced by TheyKilledKeny with this new function to convert to and from uLong <-> Low and High byte
    Private Sub Long2Int(ByVal UnsignedLong As ULong, ByRef LongLow As UInteger, ByRef LongHigh As UInteger)
        '&HFFFFFFFF unsigned = 4294967295
        LongHigh = (UnsignedLong And &HFFFFFFFF00000000) / &H100000000
        LongLow = UnsignedLong And &HFFFFFFFF
    End Sub

    Private Sub Int2Long(ByVal LongLow As UInteger, ByVal LongHigh As UInteger, ByRef UnsignedLong As ULong)
        UnsignedLong = (LongHigh * &H100000000) Or LongLow
    End Sub



End Class